%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%This is a example for the optimization of Elementary Mode Coefficients %
%%(EMCs) by Maximum Entropy Principle with Lagrange multiplier method    % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function optimization
clear all
global nd numberem nem fluxv
% ems: whole matrix of elementary mode
% uptakeflux: the flux for uptake reaction
% numuptake: the row in elementary mode matrix for uptake flux
% flux: flux for each reaction
% enzprofile:enzyme activity profile
load MEPLM_optimization
tic
disp('Preparing for optimization matrix')
[ranka,rankb]=rref(ems');
nd=rank(ems);
for i=1:1:nd
    emmatrix(i,:)=ems(rankb(1,i),:);
    fluxv(1,i)=flux(1,rankb(1,i));
end
    
[numberr,numberem]=size(ems);
% numberr:the number of reactions
% numberem: number of Elementary mode

%Normalization of EM matrix of optimization
uptakeb=ems(numuptake,:);       %%%
for i=1:1:numberem              %%%
    if uptakeb(1,i)==0          %%%
        uptakeb(1,i)=1;         %%%
    end                         %%%
end                             %%%
for j=1:1:nd
    nem(j,:)=emmatrix(j,:)*uptakeflux./uptakeb;
    for i=1:1:numberem
        if ems(numuptake,i)==0
            nem(j,i)=0;
        end
    end
end


disp('optimization is beginning')

x0=zeros(nd,1);
[x,fval]=mmfsolve(@func,x0)

zzz=0;
sss=zeros(numberem,1);
for i=1:1:numberem
    for j=1:1:nd
        sss(i,1)=sss(i,1)+x(j,1)*nem(j,i);
    end
        zzz=zzz+exp(-sss(i,1));
end

fai=exp(-sss)/zzz;
EMC=fai*uptakeflux./uptakeb';
for i=1:1:numberem
    if ems(numuptake,i)==0
    EMC(i,1)=eps;
    end
end

fluxdistribution=ems*EMC;
disp('Flux distribution is: ')
disp(fluxdistribution)

save('emc.mat','EMC');
disp('Optimization is successful. You can close it now.')
toc

function f=func(y)
global nd numberem nem fluxv
z=0;
for i=1:1:numberem
    s=0;
    for j=1:1:nd
        s=s+y(j)*nem(j,i);
    end
        z=z+exp(-s);
end
za=zeros(nd,1);
for k=1:1:nd
for i=1:1:numberem
    s=0;
    for j=1:1:nd
        s=s+y(j)*nem(j,i);
    end
        za(k,1)=za(k,1)+nem(k,i)*exp(-s);
end
f(k,1)=za(k,1)/z-fluxv(1,k);
end
disp(f)

